"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.concurrencyLimitMiddleware = void 0;
const error_1 = require("../error");
const unified_responder_1 = require("../unified-responder");
class LocalCounter {
    count;
    constructor(initialValue) {
        this.count = initialValue;
    }
    increment(val) {
        this.count += val;
    }
    decrement(val) {
        this.count -= val;
    }
    get() {
        return this.count;
    }
}
/**
 * Try to limit the number of instantaneous concurrent requests
 * @param limit The maximum number of concurrent requests. <= 0 means no limit
 * @param counter A counter object that implements the ICounter interface. If not provided, a LocalCounter will be used
 * @returns
 */
const concurrencyLimitMiddleware = (limit, counter) => {
    const c = counter || new LocalCounter(0);
    return async (ctx, next) => {
        // concurrency limit is disabled
        if (limit <= 0) {
            return next();
        }
        c.increment(1);
        if (c.get() > limit) {
            c.decrement(1);
            const err = new Error('Server is busy, please try again later');
            (0, unified_responder_1.sendResponse)(ctx, undefined, (0, error_1.newTooManyRequestsErr)(err), 429);
            return;
        }
        try {
            await next();
        }
        finally {
            c.decrement(1);
        }
    };
};
exports.concurrencyLimitMiddleware = concurrencyLimitMiddleware;
//# sourceMappingURL=concurrency-limit.js.map